/*
 * ShakeFive.cpp
 *
 *  Created on: March, 2016
 *      Author: anonymous ECCV submission 1747
 */

#include "ShakeFive.h"

#include <opencv2/core/mat.hpp>
#include <opencv2/core/operations.hpp>
#include <stddef.h>
#include <cstdlib>
#include <iostream>
#include <iterator>
#include <memory>
#include <sstream>

#include "Video.h"

using namespace nl_uu_science_gmt;

namespace nl_uu_science_gmt
{

ShakeFive::ShakeFive(
		const std::string &metadata_filename) :
				m_metadata_filename(metadata_filename)
{
}

ShakeFive::~ShakeFive()
{
}

std::vector<std::string> ShakeFive::explode(
		const std::string &sep, std::string input)
{
	std::vector<std::string> output;
	size_t found = input.find(sep);

	while (found != std::string::npos)
	{
		output.push_back(input.substr(0, found));
		input = input.substr(found + sep.length());
		found = input.find(sep);
	}
	if (input.length() > 0)
		output.push_back(input);

	return output;
}

std::string ShakeFive::implode(
		const std::string &glue, const std::vector<std::string> &parts)
{
	std::stringstream output;
	auto piece = parts.begin();
	for (; piece != parts.end(); ++piece)
		output << *piece << glue;

	return output.str().substr(0, output.str().size() - glue.size());
}

void ShakeFive::help(
		const std::string &bin_name, const std::string &example)
{
	std::cout << "Usage: " << bin_name << " <path to xml-file> [<path to xml-file>] ..." << std::endl;
	std::cout << "Trying to play: " << example << std::endl << std::endl;

	std::cout << "Controls: <ESC> - quit" << std::endl;
	std::cout << "          p     - pause playback" << std::endl;
	std::cout << "          n     - next video (if applicable)" << std::endl << std::endl;
}

int ShakeFive::play()
{
	auto path_parts = explode("/", m_metadata_filename);
	path_parts.erase(path_parts.end() - 1, path_parts.end());
	auto path_infix = implode("/", path_parts);
	auto pre_sep = "";
	if (m_metadata_filename.substr(0, 1) == "/" && path_infix.substr(0, 1) != "/")
		pre_sep = "/";
	std::string video_path = pre_sep + path_infix + "/";

	std::string filename = m_metadata_filename;
	filename.replace(0, video_path.length(), "");
	auto file_parts = explode(".", filename);
	file_parts.erase(file_parts.end() - 1, file_parts.end());
	filename = implode("", file_parts);

	Video video;
	video.init(0);
	video.setVideoPath(video_path);

	cv::FileStorage fs(m_metadata_filename, cv::FileStorage::READ);
	fs["video"] >> video;
	video.setVideoMetadataFilename(video_path + filename);
	fs.release();

	return video.play();
}

} /* namespace nl_uu_science_gmt */

int main(
		int argc, char** argv)
{
	if (argc < 2)
	{
		ShakeFive::help(argv[0], "Video-17112014-024110.xml");
	}
	else
	{
		ShakeFive shake_five(argv[1]);
		shake_five.play();
	}

	return EXIT_SUCCESS;
}
