/*
 * Skeleton.h
 *
 *  Created on: March, 2016
 *      Author: anonymous ECCV submission 1747
 */

#ifndef SKELETON_H_
#define SKELETON_H_

#include <opencv2/core/core.hpp>
#include <opencv2/core/operations.hpp>
#include <cstdbool>
#include <map>
#include <memory>
#include <set>
#include <utility>
#include <vector>

#include "Types.h"
#include "IO.h"

namespace nl_uu_science_gmt
{

class Skeleton
{
	UINT m_id;UINT m_index;
	SSkelPtr m_self;

	UINT m_fid;
	const Frame* m_frame;

	ShakeFiveVersion m_version;
	ACTION_CLASSES_INDEX m_action_class;

	cv::Rect m_bbox;
	cv::Point3d m_tlb;
	cv::Point3d m_brf;

	std::map<NUI_SKELETON_POSITION_INDEX, cv::Point2d> m_points2D;
	std::map<NUI_SKELETON_POSITION_INDEX, cv::Point3d> m_points3D;

	bool m_flipped;UINT m_screen_width;
	std::map<Kinect2JointType, cv::Point2d> m_k2_points2D;
	std::map<Kinect2JointType, cv::Point3d> m_k2_points3D;
	std::set<Kinect2JointType> m_k2_inferred_joints;

	double m_angle;

	MatPtr m_rotation;
	MatPtr m_translation;
	MatPtr m_camera_matrix;
	MatPtr m_dist_coeff;

	static inline const JointNameMap initJointNames()
	{
		JointNameMap jointNameStringMapping;
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_HIP_CENTER, "hip"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_SPINE, "spine"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_SHOULDER_CENTER, "shoulder"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_HEAD, "head"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_SHOULDER_LEFT, "left_shoulder"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_ELBOW_LEFT, "left_elbow"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_WRIST_LEFT, "left_wrist"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_HAND_LEFT, "left_hand"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_SHOULDER_RIGHT, "right_shoulder"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_ELBOW_RIGHT, "right_elbow"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_WRIST_RIGHT, "right_wrist"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_HAND_RIGHT, "right_hand"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_HIP_LEFT, "left_hip"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_KNEE_LEFT, "left_knee"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_ANKLE_LEFT, "left_ankle"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_FOOT_LEFT, "left_foot"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_HIP_RIGHT, "right_hip"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_KNEE_RIGHT, "right_knee"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_ANKLE_RIGHT, "right_ankle"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_FOOT_RIGHT, "right_foot"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_COUNT, "_count_"));

		return jointNameStringMapping;
	}

	static inline const Kinect2JointNameMap initKinect2JointNames()
	{
		Kinect2JointNameMap jointNameStringMapping;
		jointNameStringMapping.insert(std::make_pair(JointType_K2_AnkleLeft, "AnkleLeft"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_AnkleRight, "AnkleRight"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_ElbowLeft, "ElbowLeft"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_ElbowRight, "ElbowRight"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_FootLeft, "FootLeft"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_FootRight, "FootRight"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_HandLeft, "HandLeft"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_HandRight, "HandRight"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_HandTipLeft, "HandTipLeft"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_HandTipRight, "HandTipRight"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_Head, "Head"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_HipLeft, "HipLeft"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_HipRight, "HipRight"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_KneeLeft, "KneeLeft"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_KneeRight, "KneeRight"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_Neck, "Neck"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_ShoulderLeft, "ShoulderLeft"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_ShoulderRight, "ShoulderRight"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_SpineBase, "SpineBase"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_SpineMid, "SpineMid"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_SpineShoulder, "SpineShoulder"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_ThumbLeft, "ThumbLeft"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_ThumbRight, "ThumbRight"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_WristLeft, "WristLeft"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_WristRight, "WristRight"));
		jointNameStringMapping.insert(std::make_pair(JointType_K2_Count, "_count_"));

		return jointNameStringMapping;
	}

	static inline const ActionNameMap initActionNames()
	{
		ActionNameMap actionNameStringMapping;

		actionNameStringMapping.insert(std::make_pair(AC_APPROACHING, "approach"));
		actionNameStringMapping.insert(std::make_pair(AC_END, "END"));
		actionNameStringMapping.insert(std::make_pair(AC_FIST_BUMPING, "fist_bump"));
		actionNameStringMapping.insert(std::make_pair(AC_HANDSHAKING, "hand_shake"));
		actionNameStringMapping.insert(std::make_pair(AC_HIGHFIVING, "high_five"));
		actionNameStringMapping.insert(std::make_pair(AC_HUGGING, "hug"));
		actionNameStringMapping.insert(std::make_pair(AC_KISSING, "kiss"));
		actionNameStringMapping.insert(std::make_pair(AC_RPS, "rock_paper_scissors"));
		actionNameStringMapping.insert(std::make_pair(AC_TU, "thumbs_up"));
		actionNameStringMapping.insert(std::make_pair(AC_EXPL, "explain_route"));
		actionNameStringMapping.insert(std::make_pair(AC_LEAVING, "leave"));
		actionNameStringMapping.insert(std::make_pair(AC_STANDING, "stand"));
		actionNameStringMapping.insert(std::make_pair(AC_NA, "NA"));
		actionNameStringMapping.insert(std::make_pair(AC_OBJECT_PASSING, "pass_object"));
		actionNameStringMapping.insert(std::make_pair(AC_KICKING, "kick"));
		actionNameStringMapping.insert(std::make_pair(AC_POINTING, "point"));
		actionNameStringMapping.insert(std::make_pair(AC_PUNCHING, "punch"));
		actionNameStringMapping.insert(std::make_pair(AC_PUSHING, "push"));
		actionNameStringMapping.insert(std::make_pair(AC_HANDWAVING, "hand_wave"));
		actionNameStringMapping.insert(std::make_pair(AC_BOXING, "boxing"));
		actionNameStringMapping.insert(std::make_pair(AC_BOWING, "bow"));
		actionNameStringMapping.insert(std::make_pair(AC_PATTING, "pat"));
		actionNameStringMapping.insert(std::make_pair(AC_COUNT, "_count_"));

		return actionNameStringMapping;
	}

	static inline const Kinect12Map initKinect12Map()
	{
		Kinect12Map kinect12TypeMap;

		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_HIP_CENTER, JointType_K2_SpineBase));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_HIP_CENTER, JointType_K2_SpineBase));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_SPINE, JointType_K2_SpineMid));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_SHOULDER_CENTER, JointType_K2_SpineShoulder));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_HEAD, JointType_K2_Head));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_SHOULDER_LEFT, JointType_K2_ShoulderLeft));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_ELBOW_LEFT, JointType_K2_ElbowLeft));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_WRIST_LEFT, JointType_K2_WristLeft));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_HAND_LEFT, JointType_K2_HandLeft));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_SHOULDER_RIGHT, JointType_K2_ShoulderRight));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_ELBOW_RIGHT, JointType_K2_ElbowRight));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_WRIST_RIGHT, JointType_K2_WristRight));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_HAND_RIGHT, JointType_K2_HandRight));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_HIP_LEFT, JointType_K2_HipLeft));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_KNEE_LEFT, JointType_K2_KneeLeft));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_ANKLE_LEFT, JointType_K2_AnkleLeft));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_FOOT_LEFT, JointType_K2_FootLeft));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_HIP_RIGHT, JointType_K2_HipRight));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_KNEE_RIGHT, JointType_K2_KneeRight));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_ANKLE_RIGHT, JointType_K2_AnkleRight));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_FOOT_RIGHT, JointType_K2_FootRight));
		kinect12TypeMap.emplace_back(std::make_pair(SKELETON_POSITION_COUNT, JointType_K2_Count));

		return kinect12TypeMap;
	}

	static inline void drawBone(
			const std::map<NUI_SKELETON_POSITION_INDEX, cv::Point2d> &, const NUI_SKELETON_POSITION_INDEX &, const NUI_SKELETON_POSITION_INDEX &,
			const cv::Scalar &, cv::Mat &);
	static inline void drawTorso(
			const std::map<NUI_SKELETON_POSITION_INDEX, cv::Point2d> &, const cv::Scalar &, cv::Mat &);
	static inline void drawLeftArm(
			const std::map<NUI_SKELETON_POSITION_INDEX, cv::Point2d> &, const cv::Scalar &, cv::Mat &);
	static inline void drawRightArm(
			const std::map<NUI_SKELETON_POSITION_INDEX, cv::Point2d> &, const cv::Scalar &, cv::Mat &);
	static inline void drawLeftLeg(
			const std::map<NUI_SKELETON_POSITION_INDEX, cv::Point2d> &, const cv::Scalar &, cv::Mat &);
	static inline void drawRightLeg(
			const std::map<NUI_SKELETON_POSITION_INDEX, cv::Point2d> &, const cv::Scalar &, cv::Mat &);
	static inline void drawSkeleton(
			const std::map<NUI_SKELETON_POSITION_INDEX, cv::Point2d> &, const cv::Scalar &, cv::Mat &);

	void drawK2Bone(
			const std::map<Kinect2JointType, cv::Point2d> &, const Kinect2JointType &, const Kinect2JointType &, const cv::Scalar &, cv::Mat &);
	void drawK2Torso(
			const std::map<Kinect2JointType, cv::Point2d> &, const cv::Scalar &, cv::Mat &);
	void drawK2LeftArm(
			const std::map<Kinect2JointType, cv::Point2d> &, const cv::Scalar &, cv::Mat &);
	void drawK2RightArm(
			const std::map<Kinect2JointType, cv::Point2d> &, const cv::Scalar &, cv::Mat &);
	void drawK2LeftLeg(
			const std::map<Kinect2JointType, cv::Point2d> &, const cv::Scalar &, cv::Mat &);
	void drawK2RightLeg(
			const std::map<Kinect2JointType, cv::Point2d> &, const cv::Scalar &, cv::Mat &);
	void drawK2Skeleton(
			const std::map<Kinect2JointType, cv::Point2d> &, const cv::Scalar &, cv::Mat &);

public:
	static const JointNameMap JointNames;
	static const Kinect2JointNameMap Kinect2JointNames;
	static const ActionNameMap ActionNames;
	static const Kinect12Map KinectSynonyms;

	void init(
			const int, const SSkelPtr &);

	void write(
			cv::FileStorage &) const;
	void read(
			const cv::FileNode &);

	void drawSkeleton(
			const cv::Scalar &, const cv::Scalar &, cv::Mat &, ShakeFiveVersion = ShakeFiveVersion_Kinect2);
	double calcDirection();

	const UINT getId() const
	{
		return m_id;
	}

	void setId(
			UINT id)
	{
		m_id = id;
	}

	const UINT getFid() const
	{
		return m_fid;
	}

	void setFid(
			UINT fid)
	{
		m_fid = fid;
	}

	double getAngle() const
	{
		return m_angle;
	}

	void setAngle(
			double angle)
	{
		m_angle = angle;
	}

	const cv::Rect& getBbox() const
	{
		return m_bbox;
	}

	void setBbox(
			const cv::Rect& bbox)
	{
		m_bbox = bbox;
	}

	const cv::Point3d& getBrf() const
	{
		return m_brf;
	}

	void setBrf(
			const cv::Point3d& brf)
	{
		m_brf = brf;
	}

	const MatPtr& getCameraMatrix() const
	{
		return m_camera_matrix;
	}

	void setCameraMatrix(
			const MatPtr& cameraMatrix)
	{
		m_camera_matrix = cameraMatrix;
	}

	const MatPtr& getDistCoeff() const
	{
		return m_dist_coeff;
	}

	void setDistCoeff(
			const MatPtr& distCoeff)
	{
		m_dist_coeff = distCoeff;
	}

	const std::map<NUI_SKELETON_POSITION_INDEX, cv::Point2d>& getPoints2D() const
	{
		return m_points2D;
	}

	void setPoints2D(
			const std::map<NUI_SKELETON_POSITION_INDEX, cv::Point2d>& points2D)
	{
		m_points2D = points2D;
	}

	const std::map<NUI_SKELETON_POSITION_INDEX, cv::Point3d>& getPoints3D() const
	{
		return m_points3D;
	}

	void setPoints3D(
			const std::map<NUI_SKELETON_POSITION_INDEX, cv::Point3d>& points3D)
	{
		m_points3D = points3D;
	}

	const std::map<Kinect2JointType, cv::Point2d>& getK2Points2D() const
	{
		return m_k2_points2D;
	}

	void setK2Points2D(
			const std::map<Kinect2JointType, cv::Point2d>& points2D)
	{
		m_k2_points2D = points2D;
	}

	const std::map<Kinect2JointType, cv::Point3d>& getK2Points3D() const
	{
		return m_k2_points3D;
	}

	void setK2Points3D(
			const std::map<Kinect2JointType, cv::Point3d>& points3D)
	{
		m_k2_points3D = points3D;
	}

	const MatPtr& getRotation() const
	{
		return m_rotation;
	}

	void setInferredJoints(
			const std::set<Kinect2JointType> &joints)
	{
		m_k2_inferred_joints = joints;
	}

	const std::set<Kinect2JointType>& getInferredJoints() const
	{
		return m_k2_inferred_joints;
	}

	void setRotation(
			const MatPtr& rotation)
	{
		m_rotation = rotation;
	}

	const cv::Point3d& getTlb() const
	{
		return m_tlb;
	}

	void setTlb(
			const cv::Point3d& tlb)
	{
		m_tlb = tlb;
	}

	const MatPtr& getTranslation() const
	{
		return m_translation;
	}

	void setTranslation(
			const MatPtr& translation)
	{
		m_translation = translation;
	}

	const ACTION_CLASSES_INDEX getActionClass() const
	{
		return m_action_class;
	}

	void setActionClass(
			ACTION_CLASSES_INDEX actionClass)
	{
		m_action_class = actionClass;
	}

	bool isFlipped() const
	{
		return m_flipped;
	}

	void setFlipped(
			bool flipped)
	{
		m_flipped = flipped;
	}

	const UINT getScreenWidth() const
	{
		return m_screen_width;
	}

	void setScreenWidth(
			UINT sw)
	{
		m_screen_width = sw;
	}

	const Frame* getFrame() const
	{
		return m_frame;
	}

	void setFrame(
			const Frame* frame)
	{
		m_frame = frame;
	}
};

} /* namespace nl_uu_science_gmt */

#endif /* SKELETON_H_ */
