/*
 * Video.h
 *
 *  Created on: March, 2016
 *      Author: anonymous ECCV submission 1747
 */

#ifndef VIDEO_H_
#define VIDEO_H_

#include <opencv2/core/core.hpp>
#include <opencv2/core/operations.hpp>
#include <cstdbool>
#include <map>
#include <memory>
#include <string>
#include <vector>

#include "Types.h"
#include "IO.h"

namespace nl_uu_science_gmt
{

template<typename tPair>
struct second_t
{
	typename tPair::second_type operator()(
			const tPair& p) const
	{
		return p.second;
	}
};

template<typename tMap>
second_t<typename tMap::value_type> second(
		const tMap& m)
{
	return second_t<typename tMap::value_type>();
}

class Video
{
	UINT m_id;
	ShakeFiveVersion m_version;
	std::string m_video_path;
	std::string m_video_metadata_filename;
	std::string m_video_filename;
	std::string m_image_path;

	Frames m_frames;
	std::shared_ptr<cv::VideoCapture> m_video_capture;
	std::shared_ptr<cv::VideoWriter> m_video_writer;
	MatPtr m_camera_matrix;
	MatPtr m_dist_coeff;
	cv::Size m_size;
	UINT m_stride;

	bool m_paused;
	bool m_stop;
	bool m_next;
	int m_current_fid;
	int m_fps;

	std::string m_video_type;
	std::string m_video_codec;
	std::string m_video_pix_fmt;
	bool m_video_data_flipped;

	std::string m_image_type;
	bool m_image_data_flipped;

	bool m_meta_flipped;

	std::map<UINT, cv::Scalar> m_skel_colors;

	cv::Point3d m_tlb;
	cv::Point3d m_brf;

	void keyboard(
			const int);
	void initSkelColors(
			const SFramePtr &);

	void setup();

public:
	void init(const int);

	void addFrame(
			SFramePtr &);

	void write(
			cv::FileStorage &) const;
	void read(
			const cv::FileNode &);

	void createVideoCapture(
			const std::string &);
	void createVideoWriter(
			const std::string &, const int);
	void writeFrame(
			const cv::Mat &);

	int play();

	static inline void putStrokedText(
			cv::Mat& img, const std::string& text, cv::Point org, int fontFace, double fontScale, cv::Scalar color, int thickness = 1,
			int lineType = 8,
			bool bottomLeftOrigin = false)
	{
		cv::putText(img, text, org, fontFace, fontScale, Color_BLACK, thickness * 2, lineType, bottomLeftOrigin);
		cv::putText(img, text, org, fontFace, fontScale, color, thickness, lineType, bottomLeftOrigin);
	}

	const Frames& getFrames() const
	{
		return m_frames;
	}

	void setFrames(
			const Frames& frames)
	{
		m_frames = frames;
	}

	const std::shared_ptr<cv::VideoCapture>& getVideoCapture() const
	{
		return m_video_capture;
	}

	const MatPtr& getCameraMatrix() const
	{
		return m_camera_matrix;
	}

	void setCameraMatrix(
			const MatPtr& cameraMatrix)
	{
		m_camera_matrix = cameraMatrix;
	}

	const MatPtr& getDistCoeff() const
	{
		return m_dist_coeff;
	}

	void setDistCoeff(
			const MatPtr& distCoeff)
	{
		m_dist_coeff = distCoeff;
	}

	UINT getId() const
	{
		return m_id;
	}

	void setId(
			UINT id)
	{
		m_id = id;
	}

	const std::string& getVideoFilename() const
	{
		return m_video_filename;
	}

	void setVideoFilename(
			const std::string& videoFilename)
	{
		m_video_filename = videoFilename;
	}

	const std::string& getVideoMetadataFilename() const
	{
		return m_video_metadata_filename;
	}

	void setVideoMetadataFilename(
			const std::string& videoMetadataFilename)
	{
		m_video_metadata_filename = videoMetadataFilename;
	}

	const std::string& getVideoPath() const
	{
		return m_video_path;
	}

	void setVideoPath(
			const std::string& videoPath)
	{
		m_video_path = videoPath;
	}

	const cv::Size& getSize() const
	{
		return m_size;
	}

	void setSize(
			const cv::Size& size)
	{
		m_size = size;
	}

	const std::string& getImagePath() const
	{
		return m_image_path;
	}

	void setImagePath(
			const std::string& imagePath = "")
	{
		m_image_path = imagePath;
	}

	UINT getStride() const
	{
		return m_stride;
	}

	void setStride(
			UINT stride)
	{
		m_stride = stride;
	}

	bool isFlipped() const
	{
		return m_video_data_flipped;
	}

	void setFlipped(
			bool flipped)
	{
		m_video_data_flipped = flipped;
	}

	ShakeFiveVersion getVersion() const
	{
		return m_version;
	}

	void setVersion(
			ShakeFiveVersion version)
	{
		m_version = version;
	}

	bool isMetaFlipped() const
	{
		return m_meta_flipped;
	}

	void setMetaFlipped(
			bool metaFlipped)
	{
		m_meta_flipped = metaFlipped;
	}
};

} /* namespace nl_uu_science_gmt */

#endif /* VIDEO_H_ */
